<?php
namespace Teknisa\MYO\Controllers;

use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DataSource\FilterCriteria; 

class RequisicaoPedidos extends \Teknisa\GER\Controllers\TeknisaCrud {
    
    protected $crudService;
    protected $parameterBag;
    protected $entityManager;
    protected $dataSourceManager;
    
    public function __construct($crudService, ParameterBag $parameterBag, $entityManager) {
        parent::__construct($crudService, $parameterBag);

        $this->entityManager = $entityManager;
        $this->dataSourceName = "prevreti";
    }

    public function saveRateio(DTO\Request $request, DTO\Response $response) {
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            $rows = $request->getDataSet()->getRows();
            foreach ($rows as $row) {
                $servicos = $this->getServicosDestinos($row);

                if (count($servicos)) {
                    $newServicos = $this->applyRateio($row, $servicos);
                    $this->effectSave($newServicos, $response);
                }
            }
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollback();
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    private function getServicosDestinos($row) {
        
        $conditions = array(
            array(
                "columnName" => "NRPEDIDOSERV",
                "operator"   => '=',
                "value"      => $row['NRPEDIDOSERV']
            ),         
            array(
                "columnName" => "CDPRODUTO",
                "operator"   => '=',
                "value"      => $row['CDPRODUTO']
            )
        );
        $dataSet = $this->crudService->getFilteredDataSet("itempedidoserv_prevreti_servico", $conditions);
        $rows = $dataSet->getRows();
        return empty($rows) ? [] : $rows;
    }

    private function effectSave($destinos, $response){
        $destinosSave = $this->fixesColumnsToSave($destinos);
        $dataSet = new DataSet($this->dataSourceName, $destinosSave);
        $request = new DTO\Request\DataSet($dataSet, '','','');
        $rowToSave = $this->crudService->getRowsToSave($request, $this->dataSourceName);
        $this->crudService->save($this->dataSourceName, $rowToSave);
    }
    private function fixesColumnsToSave($destinos){
        foreach ($destinos as &$row) {
            $row['__is_new'] = false;
            $row['changed'] = true;
        }
        return $destinos;
    }

    private function applyRateio($row, $servicos) {
        $newServicos = $servicos;

        $QTPREVRETI   = $row["QTPREVRETI"];

        $oldQTPREVEXTRA = isset($row["changed_QTPREVEXTRA"]) ? $row["changed_QTPREVEXTRA"] : $row["QTPREVEXTRA"];
        $oldQTPREVDEVOL = isset($row["changed_QTPREVDEVOL"]) ? $row["changed_QTPREVDEVOL"] : $row["QTPREVDEVOL"];
        $oldQTPREVLIB   = isset($row["changed_QTPREVLIB"]) ? $row["changed_QTPREVLIB"] : $row["QTPREVLIB"];

        $newQTPREVEXTRA = $row["QTPREVEXTRA"];
        $newQTPREVDEVOL = $row["QTPREVDEVOL"];
        $newQTPREVLIB   = $row["QTPREVLIB"];

        $sumQTPREVRETI = $this->getSum($servicos, "QTPREVRETI");

        if ($sumQTPREVRETI == 0) {
            $newServicos = $this->rateiaIgualmente($newServicos, $newQTPREVEXTRA, $newQTPREVDEVOL);
        } else {

            $newServicos = $this->rateiaProporcionalmente($newServicos, $newQTPREVEXTRA, $newQTPREVDEVOL, $QTPREVRETI);
        }

        $newServicos = $this->calcQTPREVLIB($newServicos);
        $newServicos = $this->roundValues($newServicos);

        return $newServicos;
    }

    /**
     * Rateia os valores igualmente entre todos os serviços
     *
     * @param [Array] $servicos
     * @param [Float] $newQTPREVEXTRA
     * @param [Float] $newQTPREVDEVOL
     * @return Array Serviços rateados
     */
    private function rateiaIgualmente($servicos, $newQTPREVEXTRA, $newQTPREVDEVOL) {
        $qtServicos = count($servicos);
        foreach ($servicos as &$servico) {
            $servico["QTPREVEXTRA"] = $newQTPREVEXTRA / $qtServicos;
            $servico["QTPREVDEVOL"] = $newQTPREVDEVOL / $qtServicos;
        }
        return $servicos;
    }

    /**
     * Rateia os valores proporcionalmente de acordo com a quantidade prevista em cada serviço
     *
     * @param [Array] $servicos
     * @param [Float] $newQTPREVEXTRA
     * @param [Float] $newQTPREVDEVOL
     * @param [Float] $QTPREVRETI
     * @return Array Serviços rateados
     */
    private function rateiaProporcionalmente($servicos, $newQTPREVEXTRA, $newQTPREVDEVOL, $QTPREVRETI) {

        foreach ($servicos as &$servico) {
            $prop = $servico["QTPREVRETI"] / $QTPREVRETI;

            $servico["QTPREVEXTRA"] = $newQTPREVEXTRA * $prop;
            $servico["QTPREVDEVOL"] = $newQTPREVDEVOL * $prop;
        }

        return $servicos;
    }

    /**
     * Calcula a quantidade liberada:
     * Quantidade liberada = Quantidade Prevista + Quantidade Extra - Quantidade Devolvida
     *
     * @param [Array] $servicos
     * @return Array Serviços com a quantidade liberada calculada
     */
    function calcQTPREVLIB($servicos) {
        foreach ($servicos as &$servico) {
            $servico["QTPREVLIB"] = $servico["QTPREVEXTRA"] + $servico["QTPREVRETI"] - $servico["QTPREVDEVOL"];
        }
        return $servicos;
    }
    
    function roundValues($servicos) {
        foreach ($servicos as &$servico) {
            $servico["QTPREVEXTRA"] = round($servico["QTPREVEXTRA"], 5);
            $servico["QTPREVDEVOL"] = round($servico["QTPREVDEVOL"], 5);
            $servico["QTPREVLIB"] = round($servico["QTPREVLIB"], 5);
        }
        return $servicos;
    }

    /**
     * Retorma a soma de uma propriedade 'key' dos itens de um array
     *
     * @param [Array] $array
     * @param [String] $key
     * @return float
     */
    private function getSum($array, $key) {
        $sum = 0;
        foreach ($array as $item) {
            $sum += isset($item[$key]) ? $item[$key] : 0;
        }
        return $sum;
    }
}